<?php

// XDateTime.class.php - functions to assist when using dates and times
//
// $Id: XDateTime.class.php 1424 2008-12-29 16:09:36Z tim $
//
// (c)2000-2008 Tim Jackson (tim@timj.co.uk)
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/**
 * This class contains useful functions for dealing with dates and times,
 * especially ISO8601-formatted dates/times (i.e. dates in the 
 * format YY[YY]-MM-DD and times in the format HH:MM[:SS]). 
 * 
 * This class is primarily intended to be used uninstantiated.
 * 
 * This class is mostly unnecessary and therefore deprecated.
 */
class XDateTime {
	
	/**
	 * Split an ISO8601 date into its constituent parts
	 *
	 * @param string $isodate
	 * @return array  Array of parts, with indices "day", "month", "year", "hour", "minute" and "second"
	 */
	function SplitISODate ($isodate) {
		$date = ( substr($isodate,0,10) );
		$dateparts = explode('-',$date);
		$array['day'] = $dateparts[2];
		$array['month'] = $dateparts[1];
		$array['year'] = $dateparts[0];
		
		$timeparts = explode(':',substr($isodate,11,8));
		$array['hour'] = $timeparts[0];
		$array['minute'] = $timeparts[1];
		$array['second'] = $timeparts[2];
		
		return $array;
	}
	
	/**
	 * Get the date portion of an ISO date/time string
	 *
	 * @param string $datetime
	 * @return string
	 */
	function DateFromISODateTime ($datetime) {
		return substr($datetime,0,10);
	}

	/**
	 * Get the "time" portion of an ISO date/time string
	 *
	 * @param string $datetime
	 * @param bool $short  Whether to get hours and minutes only
	 * @return string
	 */
	function TimeFromISODateTime ($datetime,$short=false) {
		$len = 8;
		if ($short) $len = 5;
		return substr($datetime,11,$len);
	}
	
	/**
	 * Find out whether a given date is recent
	 *
	 * @param string $isodate
	 * @param int $days
	 * @param string $basedate
	 * @return bool
	 */
	function IsISODateNew ($isodate, $days, $basedate='') {
		if (!$basedate) $basedate = date('Y-m-d');
		$basetimestamp = strtotime($basedate);
		$comparetimestamp = $basetimestamp - ($days * 86400);
		$thetimestamp = strtotime($isodate);
		
		if ($thetimestamp > $comparetimestamp && $thetimestamp <= $basetimestamp) {
			return true;
		} else {
			return false;
		}
	}


	// ---------------------------------------------------------
	// DEPRECATED STUFF
	
	/**
	 * Convert an ISO8601 date to a UNIX timestamp
	 *
	 * @param string ISO 8601 date
	 * @return int UNIX timestamp
	 * @deprecated 
	 */
	function ISODate2Timestamp ($isodate) {
		return strtotime($isodate);
	}
	
	/**
	 * Convert a UNIX timestamp to an ISO8601 date
	 * 
	 * @param int $timestamp UNIX timestamp
	 * @return string ISO 8601 date
	 * @deprecated 
	 */
	function Timestamp2ISODate ($timestamp) {
		return date('Y-m-d',$timestamp);
	}

	/**
	 * Get the current date in ISO8601 format
	 *
	 * @return string
	 * @deprecated 
	 */
	function GetISODateNow () {
		return date('Y-m-d');
	}

	/**
	 * Get the current time in ISO8601 format
	 *
	 * @return string Current time
	 * @deprecated
	 */
	function GetISOTimeNow () {
		return strftime("%H:%M");
	}
}
