<?php

// DB_mysql.class.php - abstract database functions and make them easier to use
// Attempts to broadly retain syntactic compatibility with PEAR DB class.
// MySQL version

// Version $Id: DB_mysql.class.php 1800 2009-07-16 20:38:34Z tim $
//
// (c)2001-2006 Tim Jackson (tim@timj.co.uk)
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

require_once('mysql.php');

/**
 * Driver for mysql
 * 
 * Only construct via DB::connect!
 * This class shouldn't be instantiated directly, only as a result of a call to
 * DB::connect. Never do "new DB_mysql".
 *
 */
class DB_mysql {

	var $_dbh;					// Database handle
	var $_dbtype='mysql';	// Database type
	var $_dbhost;				// Database host
	var $_dbname;				// Database name

	// Connect to the database server and select the database we wish to use
	function DB_mysql ($dbhost, $username, $password, $dbname, $charset='latin1') {
		global $PHPOF_DB_CONNECTION_TYPE;
		
		if (!function_exists('mysql_connect')) {
			trigger_error('MySQL extension is not installed',E_USER_ERROR);
		}
		
		if ($PHPOF_DB_CONNECTION_TYPE == 'pconnect') {
			$this->_dbh = @mysql_pconnect($dbhost, $username, $password);
		} else {
			$this->_dbh = @mysql_connect($dbhost, $username, $password);
		}
		
		if (!$this->_dbh) {
			trigger_error('Could not connect to database ('.mysql_error($this->_dbh).')', E_USER_ERROR);
		}

		// Check if the database in use is different. if so force new connection
		$result_tmp = @mysql_query ("SELECT DATABASE();", $this->_dbh);
		$status = @mysql_fetch_row($result_tmp);
		// if database is already set then we have stolen the connection and need to create a new connection
		if (isset($status[0]) && strcasecmp($status[0],$dbname)!==0 ) {
			// force new connection
			$this->_dbh = @mysql_connect($dbhost, $username, $password, true);
		}
		
		if ($this->_dbh) {
			mysql_select_db ($dbname, $this->_dbh) or trigger_error('Could not select database',E_USER_ERROR);
			$this->_dbname = $dbname;
			$this->_dbhost = $dbhost;
			
			// Handle client character set
			if (version_compare(PHP_VERSION, '5.2.3', '>=')) {
				mysql_set_charset($charset, $this->_dbh);
			} else {
				mysql_query("SET NAMES $charset", $this->_dbh);
			}
		} else {
			trigger_error('Could not connect to database ('.mysql_error($this->_dbh).')',E_USER_ERROR);
		}
	}

	/**
	 * Perform an SQL query
	 *
	 * @param string $query  SQL query
	 * @return DB_result  Result set from the query
	 * * @see DB_mysql::query_paged()
	 */
	function query ($query) {
		global $PHPOF_DEBUG_MODE;

		if ($PHPOF_DEBUG_MODE) phpof_debug("DB_mysql::query(): $query\n");

		$microtime_start = microtime();
		$result = @mysql_query ($query, $this->_dbh);
		$microtime_end = microtime();
		list($usec_start, $sec_start) = explode(' ', $microtime_start);
		list($usec_end, $sec_end) = explode(' ', $microtime_end);
		$start = $sec_start + $usec_start;
		$end = $sec_end + $usec_end;
		$query_time = ($end-$start)*1000;
		
		if ($PHPOF_DEBUG_MODE) phpof_debug("DB_mysql::query(): $query [time=${query_time}us]\n");
		
		// Sometimes $result can be false (due to, e.g. syntax error) but mysql_error() doesn't return anything
		// e.g. SELECT * FROM table WHERE <nonexistentfield>='%foo%'
		// Need to investigate this further
		/*if ($result == false) {
			trigger_error("mysql_query returned false (SQL query was: $query)",E_USER_ERROR);
		}*/
		
		if ($err = mysql_error()) {
			$err .= " (SQL query was: $query)";
			trigger_error($err,E_USER_ERROR);
		}
		
		$resobj = new DB_result($this->_dbh, $result);
		return $resobj;
	}
	
	/**
	 * Perform a query, returning results from a particular page
	 * 
	 * This function performs the given query and returns an initialised object
	 * of type DB_result. A limited set of results will be returned, based on 
	 * the values provided for results_per_page and page_number. For example, 
	 * if you perform a query which returns 100 rows, and results_per_page is 
	 * set to 10 and page_number is set to 3, the query result will contain 
	 * rows 30-39.
	 *
	 * @param string $query  SQL query
	 * @param int $results_per_page  Number of results to return per page
	 * @param int $page  Current page number of results, starting from 1
	 * @return DB_result
	 * @see DB_mysql::query()
	 */
	function query_paged ($query, $results_per_page, $page) {
		global $PHPOF_DEBUG_MODE;
		$base = $results_per_page * ($page - 1);
		$fullquery = "$query LIMIT $base, $results_per_page";
		
		if ($PHPOF_DEBUG_MODE) phpof_debug("DB_mysql::query_paged(): $fullquery\n");
		
		$result = @mysql_query ($fullquery, $this->_dbh);
		
		if ($err = mysql_error()) {
			$err .= " (SQL query was: $fullquery)";
			trigger_error($err,E_USER_ERROR);
		}
		
		$resobj = new DB_result($this->_dbh, $result);
		$resobj->pages = $this->query_num_pages($query, $results_per_page);
		
		return $resobj;
	}
	
	/**
	 * Find the number of pages of results that will be returned with a given paged query
	 *
	 * DB_xxx::query_num_pages returns the number of pages of results which the 
	 * results from query will be divided up into, based on results_per_page 
	 * results per page. 
	 * 
	 * @param string $query  SQL query
	 * @param int $results_per_page  Results to return per "page" of results
	 * @return int  Number of pages
	 * @see DB_mysql::query_paged()
	 */
	function query_num_pages ($query, $results_per_page) {
		global $PHPOF_DEBUG_MODE;
		if ($results_per_page == 0) trigger_error('DB_mysql::query_num_pages() called with zero results per page',E_USER_ERROR);
		
		if ($PHPOF_DEBUG_MODE) phpof_debug("DB_mysql::query_num_pages(): $query\n");
		$result = @mysql_query ($query, $this->_dbh);
		
		if ($err = mysql_error()) {
			$err .= " (SQL query was: $query)";
			trigger_error($err,E_USER_ERROR);
		}
		
		$resobj = new DB_result($this->_dbh, $result);
		$rows = $resobj->numRows();
		
		return ceil ($rows / $results_per_page);
	}
	
	function beginTransaction()
	{
		$this->query('BEGIN');
	}
	
	function rollbackTransaction()
	{
		$this->query('ROLLBACK');

	}
	
	function commitTransaction()
	{
		$this->query('COMMIT');
	}
	
	function qstr($String)
	{
		return mysql_real_escape_string($String);
	}
}

class DB_result {

	var $_dbh;
	var $_rr;
	var $pages=1;	// total pages for this query

	function DB_result (&$dbh, $rr) {
		$this->_dbh = &$dbh;
		$this->_rr = $rr;
	}

	// Get the number of rows in the result
	function numRows () {
		return mysql_num_rows($this->_rr);
	}

	function fetchRow () {
		return mysql_fetch_assoc($this->_rr);
	}

	function insertedId ($table_name='') {	// $table_name not used for MySQL
		return mysql_insert_id($this->_dbh);
	}
	
	function getArray ($key, $value=PHPOF_ARRAY_LINEAR) {
		while ($result = $this->fetchRow()) {
			if ($value == PHPOF_ARRAY_LINEAR) {
				$array[] = $result[$key];
			} else {
				$array[$result[$key]] = $result[$value];
			}
		}
		if (isset($array)) {
			return $array;
		}
		return false;
	}
	
	// $Index is the name of the column to be used as an index, or PHPOF_USE_KEY
	// $Expression is an expression to be evaluated in the context of a DBRow object
	function getArrayWithDN($RowObject, $Index=PHPOF_USE_KEY, $Expression=PHPOF_USE_DEFAULT_DN, $OrderBy=PHPOF_USE_DEFAULT_ORDER) {
		
		$ReturnArray = null;
		
		if ($Expression != PHPOF_USE_DEFAULT_DN) {
			$RowObject->DescriptiveNameExpression = $Expression;
		}
		
		if (empty($RowObject->DescriptiveNameExpression)) trigger_error("DB_result::getArrayDN() called with request to use DBRow DescriptiveName (DN) as result value, but DBRow object does not have a DescriptiveNameExpression defined");
		
		if ($Index == PHPOF_USE_KEY) {
			$Index = $RowObject->_table->_key;
		}
		
		while ($this->fetchIntoObject($RowObject)) {
			$ReturnArray[$RowObject->$Index] = $RowObject->DescriptiveName;
		}
		return $ReturnArray;
	}

	function fetchIntoObject (&$object_to_fill) {
		global $PHPOF_AUDIT_LOGGING;
		if (!is_object($object_to_fill)) trigger_error('DB_result::fetchIntoObject() was not passed an object',E_USER_ERROR);
		if (!empty($object_to_fill->_Hook_FetchIntoObject_pre)) eval('$object_to_fill->'.$object_to_fill->_Hook_FetchIntoObject_pre);
		
		if ($row = $this->fetchRow()) {
			foreach ($row as $field => $value) {
				$object_to_fill->$field = $value;
			}
			if ($PHPOF_AUDIT_LOGGING) $object_to_fill->_storeAuditData();
			
			// Set row object descriptive name
			$object_to_fill->setDescriptiveName();
			
			$object_to_fill->resolveBinaryFields();
			
			// Call post-hook
			if (!empty($object_to_fill->_Hook_FetchIntoObject_post)) eval('$object_to_fill->'.$object_to_fill->_Hook_FetchIntoObject_post);
			return true;
		} else {
			return false;
		}
	}

	function affectedRows () {
		return mysql_affected_rows ($this->_dbh);
	}
	
}
